-------------------------------------------------------------------------
-- FRAMES
-------------------------------------------------------------------------

-- Create a sample frame and give it a visible background color
local AtramedesShieldClicker = CreateFrame("Frame", UIParent)

local asc = AtramedesShieldClicker

local background = asc:CreateTexture("Background", "BACKGROUND")

local m = 0
local res = ((select(GetCurrentResolution(),GetScreenResolutions())))
local width = res:match("%d+")
local height = res:match("%d+",width:len()+1)

background:SetTexture(1, 1, 1, 0.05)
background:SetAllPoints()
asc:SetSize(width*m,height*m)
asc:SetScript("OnMouseDown", function() end)
asc:SetPoint("CENTER", WorldFrame, "CENTER", 0, 0)
asc:Hide()

-------------------------------------------------------------------------
-- LOCALIZATION
-------------------------------------------------------------------------

-- Most of this is taken from BigWigs, so thanks to them mostly
-- If your clients version is not localized, feel free to add it yourself

local function getLocalizedYell()
	local locale = GetLocale()
	if locale == "deDE" then
		return "Ja, lauft! Jeder Schritt lässt Euer Herz stärker klopfen. Laut und heftig... ohrenbetäubend. Es gibt kein Entkommen!" -- check
	elseif locale == "enGB" then
		return "Yes, run! With every step your heart quickens. The beating, loud and thunderous... Almost deafening. You cannot escape!"
	elseif locale == "enUS" then
		return "Yes, run! With every step your heart quickens. The beating, loud and thunderous... Almost deafening. You cannot escape!"
	elseif locale == "esES" then
		return "¡Sí, corred! Con cada paso, vuestros corazones se aceleran. El latido, fuerte y clamoroso... Casi ensordecedor. ¡No podéis escapar!"
	elseif locale == "esMX" then
		return "Yes, run! With every step your heart quickens. The beating, loud and thunderous... Almost deafening. You cannot escape!"
	elseif locale == "frFR" then
		return "Oui, fuyez ! Chaque foulée accélère votre cœur. Les battements résonnent comme le tonnerre... Assourdissant. Vous ne vous échapperez pas !" -- à vérifier
	elseif locale == "koKR" then
		return "그래, 도망가라! 발을 디딜 때마다 맥박은 빨라지지. 점점 더 크게 울리는구나... 귀청이 터질 것만 같군! 넌 달아날 수 없다!"
	elseif locale == "ruRU" then
		return "Да, беги! С каждым шагом твое сердце бьется все быстрее. Эти громкие, оглушительные удары... Тебе некуда бежать!"
	elseif locale == "zhCN" then
		return "Yes, run! With every step your heart quickens. The beating, loud and thunderous... Almost deafening. You cannot escape!"
	elseif locale == "zhTW" then
		return "沒錯，逃吧!每一步都會讓你的心跳加速。跳得轟隆作響...震耳欲聾。你逃不掉的!"
	end
end

-------------------------------------------------------------------------
-- UTILITY
-------------------------------------------------------------------------

local airPhaseDuration = 30

local function show()
	asc:Show()
end

local function initializeScale()
	if not AtramedesShieldClickerDB then
		AtramedesShieldClickerDB = {}
		AtramedesShieldClickerDB["m"] = 0.5
	end
	m = AtramedesShieldClickerDB["m"]
	asc:SetSize(width*m,height*m)
end

local function engageCheck()
	if not UnitExists("boss1") then return end
	if tonumber(UnitGUID("boss1"):sub(7, 10), 16) == 41442 then
		asc:Hide()
		asc:ScheduleTimer(show, 40)
	end
end

local function checkForWipe()
	local w = true
	local num = GetNumGroupMembers()
	for i = 1, num do
		local name = GetRaidRosterInfo(i)
		if name then
			if UnitAffectingCombat(name) then
				w = false
			end
		end
	end
	if w then
		asc:Hide()
		asc:CancelAllTimers()
	end
	if not w then asc:ScheduleTimer(checkForWipe, 2) end
end

local function groundPhase()
	-- XXX need a good trigger for ground phase start to make this even more accurate
	asc:ScheduleTimer(show, 45)
end

-------------------------------------------------------------------------
-- SLASH COMMAND
-------------------------------------------------------------------------

local function slashCommand(input)
	if input:find("scale") then
		local value = tonumber(input:match("[%d.%d]+"))
		if value > 1 then
			print("Scale has to be less than 1. default is 0.5")
		end
		AtramedesShieldClickerDB["m"], m = value, value
		asc:SetSize(width*m,height*m)
		return
	end
	if asc:IsShown() then
		asc:Hide()
	else
		asc:Show()
	end
end

-------------------------------------------------------------------------
-- EVENT HANDLING
-------------------------------------------------------------------------

local function registerCombatEvents()
	asc:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	asc:RegisterEvent("PLAYER_REGEN_ENABLED")
	asc:RegisterEvent("CHAT_MSG_MONSTER_YELL")
	asc:RegisterEvent("INSTANCE_ENCOUNTER_ENGAGE_UNIT")
end

local function unregisterCombatEvents()
	asc:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	asc:UnregisterEvent("PLAYER_REGEN_ENABLED")
	asc:UnregisterEvent("CHAT_MSG_MONSTER_YELL")
	asc:UnregisterEvent("INSTANCE_ENCOUNTER_ENGAGE_UNIT")
end

local function detectInstanceChange()
	if IsInInstance() then
		SetMapToCurrentZone()
	end
	local zone = GetRealZoneText()
	if zone == nil or zone == "" then
		-- zone hasn't been loaded yet, try again in 5 secs.
		asc:ScheduleTimer(detectInstanceChange, 5)
		return
	elseif GetCurrentMapAreaID() == 754 and IsInInstance() then
		registerCombatEvents()
	else
		unregisterCombatEvents()
		asc:Hide()
		asc:CancelAllTimers()
	end
end

asc:RegisterEvent("ADDON_LOADED")
asc:RegisterEvent("ZONE_CHANGED_NEW_AREA")

asc:SetScript("OnEvent", function(self, event, ...)
    self[event](self, ...)
end)

function asc:COMBAT_LOG_EVENT_UNFILTERED(...)
	local _, event, _, _, _, _, _, _, _, _, _, spellId = ...
	if event == "SPELL_AURA_APPLIED" then
		if spellId == 77840 then
			asc:Hide()
		end
	end
end

function asc:CHAT_MSG_MONSTER_YELL(msg)
	if msg == getLocalizedYell() then
		asc:ScheduleTimer(groundPhase, airPhaseDuration)
	end
end

function asc:PLAYER_REGEN_ENABLED()
	checkForWipe()
end

function asc:INSTANCE_ENCOUNTER_ENGAGE_UNIT()
	engageCheck()
end

function asc:ADDON_LOADED(addon)
	if addon == "AtramedesShieldClicker" then
		LibStub("AceTimer-3.0"):Embed(asc)
		detectInstanceChange()
		self:UnregisterEvent("ADDON_LOADED")
		SlashCmdList.asc = slashCommand
		SLASH_asc1 = "/asc"

		initializeScale()
	end
end

function asc:ZONE_CHANGED_NEW_AREA()
	detectInstanceChange()
end